Merge branch 'Options'
[usenet/yapfaq.git] / yapfaq.pl
index a50c08a..0a541c8 100644 (file)
--- a/yapfaq.pl
+++ b/yapfaq.pl
@@ -14,7 +14,7 @@
 
 my $Version = "0.6-unreleased";
 
-my $NNTPServer = "";
+my $NNTPServer = "localhost";
 my $NNTPUser = "";
 my $NNTPPass = "";
 my $Sender = "";
@@ -54,21 +54,32 @@ use strict;
 use Net::NNTP;
 use Date::Calc qw(Add_Delta_YM Add_Delta_Days Delta_Days Today);
 use Fcntl ':flock'; # import LOCK_* constants
+use Getopt::Std;
 my ($TDY, $TDM, $TDD) = Today(); #TD: Today's date
 
+my %Options;
+getopts('hvpdt:f:', \%Options);
+if ($Options{'h'}) {
+  print "$0 v $Version\nUsage: $0 [-hvpd] [-t <newsgroups>] [-f <faq>]\n";
+  exit(0);
+};
+my ($Faq) = $Options{'f'} if ($Options{'f'});
+
 my @Config;
-readconfig (\$ConfigFile, \@Config);
+readconfig (\$ConfigFile, \@Config, \$Faq);
 
 foreach (@Config) { 
   my ($LPD,$LPM,$LPY) = (01, 01, 0001);  #LP: Last posting-date
   my ($NPY,$NPM,$NPD);                   #NP: Next posting-date
   my $SupersedeMID;
   
-  my ($ActName,$File,$PFreq) =($$_{'name'},$$_{'file'},$$_{'posting-frequency'});
+  my ($ActName,$File,$PFreq,$Expire) =($$_{'name'},$$_{'file'},$$_{'posting-frequency'},$$_{'expires'});
   my ($From,$Subject,$NG,$Fup2)=($$_{'from'},$$_{'subject'},$$_{'ngs'},$$_{'fup2'});
   my ($MIDF,$ReplyTo,$ExtHea)=($$_{'mid-format'},$$_{'reply-to'},$$_{'extraheader'});
   my ($Supersede)            =($$_{'supersede'});
-    
+
+  next if (defined($Faq) && $ActName ne $Faq);
+       
   if (open (FH, "<$File.cfg")) {
     while(<FH>){
       if (/##;; Lastpost:\s*(\d{1,2})\.(\d{1,2})\.(\d{2}(\d{2})?)/){
@@ -79,34 +90,41 @@ foreach (@Config) {
     }
     close FH;
   } else { 
-    warn "Couldn't open $File.cfg: $!\n";
+    warn "$0: W: Couldn't open $File.cfg: $!\n";
   }
 
   $SupersedeMID = "" unless $Supersede;
 
-  if ($PFreq =~ /(\d+)\s*([dw])/) { # Is counted in days or weeks: Use Add_Delta_Days.
-    ($NPY,$NPM,$NPD) = Add_Delta_Days($LPY, $LPM, $LPD, (($2 eq "w")?$1 * 7: $1 * 1));
-  } elsif ($PFreq =~ /(\d+)\s*([my])/) { #Is counted in months or years: Use Add_Delta_YM
-    ($NPY,$NPM,$NPD) = Add_Delta_YM($LPY, $LPM, $LPD, (($2 eq "m")?(0,$1):($1,0)));
-  }
-    
-  if (Delta_Days($NPY,$NPM,$NPD,$TDY,$TDM,$TDD) >= 0 ) {
-    postfaq(\$ActName,\$File,\$From,\$Subject,\$NG,\$Fup2,\$MIDF,\$ExtHea,\$Sender,\$TDY,\$TDM,\$TDD,\$ReplyTo,\$SupersedeMID);
+  ($NPY,$NPM,$NPD) = calcdelta ($LPY,$LPM,$LPD,$PFreq);
+
+  if (Delta_Days($NPY,$NPM,$NPD,$TDY,$TDM,$TDD) >= 0 or ($Options{'p'})) {
+    if($Options{'d'}) {
+         print "$ActName: Would be posted now (but running in simulation mode [$0 -d]).\n" if $Options{'v'};
+       } else {
+      postfaq(\$ActName,\$File,\$From,\$Subject,\$NG,\$Fup2,\$MIDF,\$ExtHea,\$Sender,\$TDY,\$TDM,\$TDD,\$ReplyTo,\$SupersedeMID,\$Expire);
+       }
+  } elsif($Options{'v'}) {
+    print "$ActName: Nothing to do.\n";
   }
 }
 
 exit;
 
 ################################## readconfig ##################################
# Takes a filename and the reference to an array, which will hold hashes with
# the data from $File.
 
 sub readconfig{
-  my ($File, $Config) = @_;
+  my ($File, $Config, $Faq) = @_;
   my ($LastEntry, $Error, $i) = ('','',0);
 
-  open FH, "<$$File" or die "$0: Can't open $$File: $!";
+  if($Options{'v'}) {
+    print "Reading configuration.\n";
+  }
+
+  open FH, "<$$File" or die "$0: E: Can't open $$File: $!";
   while (<FH>) {
+    next if (defined($$Faq) && !/^\s*=====\s*$/ && defined($$Config[$i]{'name'}) && $$Config[$i]{'name'} ne $$Faq );
     if (/^(\s*(\S+)\s*=\s*'?(.*?)'?\s*(#.*$|$)|^(.*?)'?\s*(#.*$|$))/ && not /^\s*$/) {
       $LastEntry = lc($2) if $2;
       $$Config[$i]{$LastEntry} .= $3 if $3;  
@@ -120,32 +138,57 @@ sub readconfig{
 
   #Check saved values:
   for $i (0..$i){
+    next if (defined($$Faq) && defined($$Config[$i]{'name'}) && $$Config[$i]{'name'} ne $$Faq );
     unless($$Config[$i]{'from'} =~ /\S+\@(\S+\.)?\S{2,}\.\S{2,}/) {
-      $Error .= "The From-header for your project \"$$Config[$i]{'name'}\" seems to be incorrect.\n"
+      $Error .= "E: The From-header for your project \"$$Config[$i]{'name'}\" seems to be incorrect.\n"
     }
     unless($$Config[$i]{'ngs'} =~ /^\S+$/) {
-      $Error .= "The Newsgroups-header for your project \"$$Config[$i]{'name'}\" contains whitespaces.\n"
+      $Error .= "E: The Newsgroups-header for your project \"$$Config[$i]{'name'}\" contains whitespaces.\n"
     }
     unless(!$$Config[$i]{'fup2'} || $$Config[$i]{'fup2'} =~ /^\S+$/) {
-      $Error .= "The Followup-To-header for your project \"$$Config[$i]{'name'}\" contains whitespaces.\n"
+      $Error .= "E: The Followup-To-header for your project \"$$Config[$i]{'name'}\" contains whitespaces.\n"
     }
     unless($$Config[$i]{'posting-frequency'} =~ /^\s*\d+\s*[dwmy]\s*$/) {
-      $Error .= "The Posting-frequency for your project \"$$Config[$i]{'name'}\" is invalid.\n"
+      $Error .= "E: The Posting-frequency for your project \"$$Config[$i]{'name'}\" is invalid.\n"
+    }
+    unless($$Config[$i]{'expires'} =~ /^\s*\d+\s*[dwmy]\s*$/) {
+      $$Config[$i]{'expires'} = '3m'; # set default: 3 month
+         warn "$0: W: The Expires for your project \"$$Config[$i]{'name'}\" is invalid - set to 3 month.\n";
     }
     $Error .= "-" x 25 . "\n" if $Error;
   }
   die $Error if $Error;
 }
 
+################################# calcdelta #################################
+# Takes a date (year,  month and day) and a time period (1d, 1w, 1m, 1y, ...)
+# and adds the latter to the former
+
+sub calcdelta {
+  my ($Year, $Month, $Day, $Period) = @_;
+  my ($NYear, $NMonth, $NDay);
+
+  if ($Period =~ /(\d+)\s*([dw])/) { # Is counted in days or weeks: Use Add_Delta_Days.
+    ($NYear, $NMonth, $NDay) = Add_Delta_Days($Year, $Month, $Day, (($2 eq "w")?$1 * 7: $1 * 1));
+  } elsif ($Period =~ /(\d+)\s*([my])/) { #Is counted in months or years: Use Add_Delta_YM
+    ($NYear, $NMonth, $NDay) = Add_Delta_YM($Year, $Month, $Day, (($2 eq "m")?(0,$1):($1,0)));
+  }
+  return ($NYear, $NMonth, $NDay);
+}
+  
 ################################## postfaq ##################################
 # Takes a filename and many other vars.
 #
 # It reads the data-file $File and then posts the article.
 
 sub postfaq {
-  my ($ActName,$File,$From,$Subject,$NG,$Fup2,$MIDF,$ExtraHeaders,$Sender,$TDY,$TDM,$TDD,$ReplyTo,$Supersedes) = @_;
+  my ($ActName,$File,$From,$Subject,$NG,$Fup2,$MIDF,$ExtraHeaders,$Sender,$TDY,$TDM,$TDD,$ReplyTo,$Supersedes,$Expire) = @_;
   my (@Header,@Body,$MID,$InRealBody,$LastModified);
 
+  if($Options{'v'}) {
+    print "$$ActName: Preparing to post.\n";
+  }
+  
   #Prepare MID:
   $$TDM = ($$TDM < 10 && $$TDM !~ /^0/) ? "0" . $$TDM : $$TDM;
   $$TDD = ($$TDD < 10 && $$TDD !~ /^0/) ? "0" . $$TDD : $$TDD;
@@ -181,17 +224,22 @@ sub postfaq {
   my $year = (1900 + $time[5]);
   my $tz = $time[8] ? " +0200" : " +0100";
   
-  my ($expY,$expM,$expD) = Add_Delta_YM($year, $month, $day, 0, 3);
+  my ($expY,$expM,$expD) = calcdelta ($year,$month,$day,$$Expire);
   my $expmonthN = ("Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec")[$expM-1];
 
   my $date = "$day $monthN $year " . $hh . ":" . $mm . ":" . $ss . $tz;
   my $expdate = "$expD $expmonthN $expY $hh:$mm:$ss$tz";
-
   #Replace %LM by the content of the news.answer-pseudo-header Last-modified:
   if ($LastModified) {
     $$Subject =~ s/\%LM/$LastModified/;
   }
 
+  # Test mode?
+  if($Options{'t'} and $Options{'t'} !~ /console/i) {
+    $$NG = $Options{'t'};
+  }
+
   #Now create the complete Header:
   push @Header, "From: $$From\n";
   push @Header, "Newsgroups: $$NG\n";
@@ -213,9 +261,16 @@ sub postfaq {
 
   my @Article = ($UsePGP)?@{signpgp(\@Header, \@Body)}:(@Header, "\n", @Body);
   
+  if($Options{'v'}) {
+    print "$$ActName: Posting article ...\n";
+  }
   post(\@Article);
 
-  open (FH, ">$$File.cfg") or die "$0: Can't open $$File.cfg: $!";
+  if($Options{'v'}) {
+    print "$$ActName: Save status information.\n";
+  }
+
+  open (FH, ">$$File.cfg") or die "$0: E: Can't open $$File.cfg: $!";
   print FH "##;; Lastpost: $day.$month.$year\n";
   print FH "##;; LastMID: $MID\n";
   close FH;
@@ -229,8 +284,16 @@ sub postfaq {
 sub post {
   my ($ArticleR) = @_;
 
+  # Test mode?
+  if(defined($Options{'t'}) and $Options{'t'} =~ /console/i) {
+    print "\n-----BEGIN--------------------------------------------------\n";
+       print @$ArticleR;
+    print "\n------END---------------------------------------------------\n";
+       return;
+  }
+
   my $NewsConnection = Net::NNTP->new($NNTPServer, Reader => 1)
-    or die "Can't connect to news server $NNTPServer!\n";
+    or die "$0: E: Can't connect to news server '$NNTPServer'!\n";
 
   $NewsConnection->authinfo ($NNTPUser, $NNTPPass);
   $NewsConnection->post();
@@ -262,7 +325,7 @@ sub getpgpcommand {
 
   if ($PGPVersion eq '2') {
     if ($PathtoPGPPass && !$PGPPass) {
-      open (PGPPW, $PathtoPGPPass) or die "Can't open $PathtoPGPPass: $!";
+      open (PGPPW, $PathtoPGPPass) or die "$0: E: Can't open $PathtoPGPPass: $!";
       $PGPPass = <PGPPW>;
       close PGPPW;
     }
@@ -270,22 +333,22 @@ sub getpgpcommand {
     if ($PGPPass) {
       $PGPCommand = "PGPPASS=\"".$PGPPass."\" ".$pgp." -u \"".$PGPSigner."\" +verbose=0 language='en' -saft <".$pgptmpf.".txt >".$pgptmpf.".txt.asc";
     } else {
-      die "$0: PGP-Passphrase is unknown!\n";
+      die "$0: E: PGP-Passphrase is unknown!\n";
     }
   } elsif ($PGPVersion eq '5') {
     if ($PathtoPGPPass) {
       $PGPCommand = "PGPPASSFD=2 ".$pgp."s -u \"".$PGPSigner."\" -t --armor -o ".$pgptmpf.".txt.asc -z -f < ".$pgptmpf.".txt 2<".$PathtoPGPPass;
     } else {
-      die "$0: PGP-Passphrase is unknown!\n";
+      die "$0: E: PGP-Passphrase is unknown!\n";
     }
   } elsif ($PGPVersion =~ m/GPG/io) {
     if ($PathtoPGPPass) {
       $PGPCommand = $pgp." --digest-algo MD5 -a -u \"".$PGPSigner."\" -o ".$pgptmpf.".txt.asc --no-tty --batch --passphrase-fd 2 2<".$PathtoPGPPass." --clearsign ".$pgptmpf.".txt";
     } else {
-      die "$0: Passphrase is unknown!\n";
+      die "$0: E: Passphrase is unknown!\n";
     }
   } else {
-    die "$0: Unknown PGP-Version $PGPVersion!";
+    die "$0: E: Unknown PGP-Version $PGPVersion!";
   }
   return $PGPCommand;
 }
@@ -334,31 +397,31 @@ sub signpgp {
     }
   }
 
-  open(FH, ">" . $pgptmpf . ".txt") or die "$0: can't open $pgptmpf: $!\n";
+  open(FH, ">" . $pgptmpf . ".txt") or die "$0: E: can't open $pgptmpf: $!\n";
   print FH $pgphead, "\n", $pgpbody;
   print FH "\n" if ($PGPVersion =~ m/GPG/io);  # workaround a pgp/gpg incompatibility - should IMHO be fixed in pgpverify
-  close(FH) or warn "$0: Couldn't close TMP: $!\n";
+  close(FH) or warn "$0: W: Couldn't close TMP: $!\n";
 
   # Start PGP, then read the signature;
   my $PGPCommand = getpgpcommand($PGPVersion);
   `$PGPCommand`;
 
-  open (FH, "<" . $pgptmpf . ".txt.asc") or die "$0: can't open ".$pgptmpf.".txt.asc: $!\n";
+  open (FH, "<" . $pgptmpf . ".txt.asc") or die "$0: E: can't open ".$pgptmpf.".txt.asc: $!\n";
   $/ = "$pgpbegin\n";
   $_ = <FH>;
   unless (m/\Q$pgpbegin\E$/o) {
 #    unlink $pgptmpf . ".txt";
 #    unlink $pgptmpf . ".txt.asc";
-    die "$0: $pgpbegin not found in ".$pgptmpf.".txt.asc\n"
+    die "$0: E: $pgpbegin not found in ".$pgptmpf.".txt.asc\n"
   }
-  unlink($pgptmpf . ".txt") or warn "$0: Couldn't unlink $pgptmpf.txt: $!\n";
+  unlink($pgptmpf . ".txt") or warn "$0: W: Couldn't unlink $pgptmpf.txt: $!\n";
 
   $/ = "\n";
   $_ = <FH>;
   unless (m/^Version: (\S+)(?:\s(\S+))?/o) {
     unlink $pgptmpf . ".txt";
     unlink $pgptmpf . ".txt.asc";
-    die "$0: didn't find PGP Version line where expected.\n";
+    die "$0: E: didn't find PGP Version line where expected.\n";
   }
   
   if (defined($2)) {
@@ -383,7 +446,7 @@ sub signpgp {
   unless (eof(FH)) {
     unlink $pgptmpf . ".txt";
     unlink $pgptmpf . ".txt.asc";
-    die "$0: unexpected data following $pgpend\n";
+    die "$0: E: unexpected data following $pgpend\n";
   }
   close(FH);
   unlink "$pgptmpf.txt.asc";
@@ -79,34 +90,41 @@ foreach (@Config) {
 
 ################################## readconfig ##################################
-# Takes a filename and the reference to an array, which will hold hashes with
-# the data from $File.
+# Takes a filename, a reference to an array, which will hold hashes with
+# the data from $File, and - optionally - the name of the (single) FAQ to post
 
 sub readconfig{
This page took 0.015728 seconds and 4 git commands to generate.